<?php
/* Copyright (C) 2024 Open-Dsi	<support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * 	\file		htdocs/suppliercontract/class/actions_suppliercontract.class.php
 * 	\ingroup	suppliercontract
 * 	\brief		File class for actions/hooks of supplier contract
 */

require_once DOL_DOCUMENT_ROOT . '/core/class/html.formfile.class.php';
require_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
dol_include_once('/suppliercontract/class/suppliercontract.class.php');

class ActionsSupplierContract
{
	/**
	 * @var DoliDB Database handler.
	 */
	public $db;
	/**
	 * @var string Error
	 */
	public $error = '';
	/**
	 * @var array Errors
	 */
	public $errors = array();

	/**
	 * @var array Hook results. Propagated to $hookmanager->resArray for later reuse
	 */
	public $results = array();

	/**
	 * @var string String displayed by executeHook() immediately after return
	 */
	public $resprints;

	/**
	 * Constructor
	 *
	 * @param        DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		$this->db = $db;
	}

	/**
	 * Overload the addMoreActionsButtons function : replacing the parent's function with the one below
	 *
	 * @param	array<string,mixed>	$parameters		Hook metadata (context, etc...)
	 * @param	CommonObject		$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	?string				$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager			$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int									Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function addMoreActionsButtons($parameters, &$object, &$action, $hookmanager)
	{
		global $confg, $langs, $user;

		$contexts = explode(':', $parameters['context']);

		if (isModEnabled("suppliercontract") && $user->hasRight('suppliercontract', 'creer')) {
			if (in_array('thirdpartysupplier', $contexts) && $object->status == 1) {
				$langs->loadLangs(array("contracts", "suppliercontracts@suppliercontract"));
				print '<div class="inline-block divButAction"><a class="butAction" href="' . dol_buildpath('/suppliercontract/card.php', 1) . '?socid=' . $object->id . '&amp;action=create">' . $langs->trans("AddSupplierContract") . '</a></div>';
			} elseif (in_array('projectcard', $contexts) && !getDolGlobalString('PROJECT_HIDE_CREATE_OBJECT_BUTTON')) {
				$langs->loadLangs(array("contracts", "suppliercontracts@suppliercontract"));
				print '<div class="inline-block divButAction"><a class="butAction" href="' . dol_buildpath('/suppliercontract/card.php', 1) . '?action=create&amp;projectid=' . $object->id . '&amp;socid=' . $object->socid . '">' . $langs->trans("AddSupplierContract") . '</a></div>';
			} elseif (in_array('supplier_proposalcard', $contexts) && $object->statut == SupplierProposal::STATUS_SIGNED) {
				$langs->loadLangs(array("contracts", "suppliercontracts@suppliercontract"));
				print '<div class="inline-block divButAction"><a class="butAction" href="' . dol_buildpath('/suppliercontract/card.php', 1) . '?action=create&amp;origin='.$object->element.'&amp;originid='.$object->id.'&amp;socid=' . $object->id . '">' . $langs->trans("AddSupplierContract") . '</a></div>';
			} elseif (in_array('ordersuppliercard', $contexts) && ($object->statut == CommandeFournisseur::STATUS_VALIDATED || $object->statut == CommandeFournisseur::STATUS_ACCEPTED || $object->statut == CommandeFournisseur::STATUS_ORDERSENT || $object->statut == CommandeFournisseur::STATUS_RECEIVED_PARTIALLY || $object->statut == CommandeFournisseur::STATUS_RECEIVED_COMPLETELY)) {
				$langs->loadLangs(array("contracts", "suppliercontracts@suppliercontract"));
				print '<div class="inline-block divButAction"><a class="butAction" href="' . dol_buildpath('/suppliercontract/card.php', 1) . '?action=create&amp;origin='.$object->element.'&amp;originid='.$object->id.'&amp;socid=' . $object->id . '">' . $langs->trans("AddSupplierContract") . '</a></div>';
			} elseif (in_array('invoicesuppliercard', $contexts) && getDolGlobalString('CONTRACT_CREATE_FROM_SUPPLIER_INVOICE') && $object->status == FactureFournisseur::STATUS_VALIDATED) {
				$langs->loadLangs(array("contracts", "suppliercontracts@suppliercontract"));
				print '<div class="inline-block divButAction"><a class="butAction" href="' . dol_buildpath('/suppliercontract/card.php', 1) . '?socid=' . $object->id . '&amp;action=create">' . $langs->trans("AddSupplierContract") . '</a></div>';
			}
		}

		return 0;
	}

	/**
	 * Overload the addSearchEntry function : replacing the parent's function with the one below
	 *
	 * @param	array<string,mixed>	$parameters		Hook metadata (context, etc...)
	 * @param	CommonObject		$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	?string				$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager			$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int									Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function addSearchEntry($parameters, &$object, &$action, $hookmanager)
	{
		global $confg, $langs;

		$contexts = explode(':', $parameters['context']);

		$resArray = array();
		if (in_array('searchform', $contexts)) {
			$resArray['searchintosuppliercontract'] = array(
				'text' => img_picto('', 'contract') . ' ' . $langs->trans("SupplierContract", GETPOST('q')),
				'url' => dol_buildpath('/suppliercontract/list.php?sall=' . urlencode(GETPOST('q')), 1)
			);
		}

		$this->results = $resArray;
		return 0;
	}

	/**
	 * Overload the completeListOfReferent function : replacing the parent's function with the one below
	 *
	 * @param	array<string,mixed>	$parameters		Hook metadata (context, etc...)
	 * @param	CommonObject		$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	?string				$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager			$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int									Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function completeListOfReferent($parameters, &$object, &$action, $hookmanager)
	{
		global $user;

		$contexts = explode(':', $parameters['context']);

		$resArray = array();
		if (in_array('projectOverview', $contexts)) {
			$id = GETPOSTINT('id');

			$resArray['suppliercontract'] = array(
				'name' => "SupplierContracts",
				'title' => "ListSupplierContractsAssociatedProject",
				'class' => 'SupplierContract',
				'table' => 'suppliercontract',
				'datefieldname' => 'date_contract',
				'margin' => '',
				'project_field' => 'fk_project',
				'disableamount' => 0,
				'urlnew' => dol_buildpath('/suppliercontract/card.php', 1) . '?action=create&projectid=' . $id . '&backtopage=' . urlencode($_SERVER['PHP_SELF'] . '?id=' . $id),
				'lang' => 'suppliercontracts@suppliercontract',
				'buttonnew' => 'AddSupplierContract',
				'testnew' => $user->hasRight('suppliercontract', 'creer'),
				'test' => isModEnabled('suppliercontract') && $user->hasRight('suppliercontract', 'lire')
			);
		}

		$this->results = $resArray;
		return 0;
	}

	/**
	 * Overload the showLinkToObjectBlock function : replacing the parent's function with the one below
	 *
	 * @param	array<string,mixed>	$parameters		Hook metadata (context, etc...)
	 * @param	CommonObject		$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	?string				$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager			$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int									Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function showLinkToObjectBlock($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $langs;

		$contexts = explode(':', $parameters['context']);

		$possiblelinks = array();
		if (in_array('commonobject', $contexts)) {
			$listofidcompanytoscan = $parameters['listofidcompanytoscan'];
			$possiblelinks = $parameters['possiblelinks'];
			if (!is_object($object->thirdparty)) {
				$object->fetch_thirdparty();
			}

			if (is_object($object->thirdparty) && !empty($object->thirdparty->id) && $object->thirdparty->id > 0) {
				$listofidcompanytoscan = $object->thirdparty->id;
				if (($object->thirdparty->parent > 0) && getDolGlobalString('THIRDPARTY_INCLUDE_PARENT_IN_LINKTO')) {
					$listofidcompanytoscan .= ',' . $object->thirdparty->parent;
				}
				if (($object->fk_project > 0) && getDolGlobalString('THIRDPARTY_INCLUDE_PROJECT_THIRDPARY_IN_LINKTO')) {
					include_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
					$tmpproject = new Project($this->db);
					$tmpproject->fetch($object->fk_project);
					if ($tmpproject->socid > 0 && ($tmpproject->socid != $object->thirdparty->id)) {
						$listofidcompanytoscan .= ',' . $tmpproject->socid;
					}
					unset($tmpproject);
				}

				$possiblelinks = array(
					'suppliercontract' => array(
						'enabled' => isModEnabled('suppliercontract'),
						'perms' => 1,
						'label' => 'LinkToSupplierContract',
						'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref, t.ref_customer as ref_client, t.ref_supplier, SUM(td.total_ht) as total_ht
							FROM " . $this->db->prefix() . "societe as s, " . $this->db->prefix() . "suppliercontract as t, " . $this->db->prefix() . "suppliercontractdet as td WHERE t.fk_soc = s.rowid AND td.fk_suppliercontract = t.rowid AND t.fk_soc IN (" . $this->db->sanitize($listofidcompanytoscan) . ') AND t.entity IN (' . getEntity('suppliercontract') . ') GROUP BY s.rowid, s.nom, s.client, t.rowid, t.ref, t.ref_customer, t.ref_supplier'
					),
				);
			}
		}

		$this->results = $possiblelinks;
		return 0;
    }

	/**
	 * Overload the showLinkToObjectBlock function : replacing the parent's function with the one below
	 *
	 * @param	array<string,mixed>	$parameters		Hook metadata (context, etc...)
	 * @param	CommonObject		$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	?string				$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager			$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int									Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function addOpenElementsDashboardLine($parameters, &$object, &$action, $hookmanager)
	{
		global $conf, $langs, $user;

		$contexts = explode(':', $parameters['context']);

		$dashboardlines = array();
		if (in_array('index', $contexts)) {
			// Number of supplier contract / services enabled (delayed)
			if (isModEnabled('suppliercontract') && !getDolGlobalString('MAIN_DISABLE_BLOCK_SUPPLIERCONTRACT') && $user->hasRight('suppliercontract', 'lire')) {
				dol_include_once('/suppliercontract/class/suppliercontract.class.php');
				$board = new SupplierContract($this->db);
				$dashboardlines[$board->element . '_inactive'] = $board->load_board($user, "inactive");
				// Number of active services (expired)
				$dashboardlines[$board->element . '_active'] = $board->load_board($user, "active");
			}
		}

		$this->results = $dashboardlines;
		return 0;
	}

	/**
	 * Overload the addOpenElementsDashboardGroup function : replacing the parent's function with the one below
	 *
	 * @param	array<string,mixed>	$parameters		Hook metadata (context, etc...)
	 * @param	CommonObject		$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	?string				$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager			$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int									Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function addOpenElementsDashboardGroup($parameters, &$object, &$action, $hookmanager)
	{
		global $conf, $langs, $user;

		$contexts = explode(':', $parameters['context']);

		$dashboardgroup = array();
		if (in_array('index', $contexts)) {
			$dashboardgroup['suppliercontract'] = array(
				'groupName' => 'SupplierContracts',
				'globalStatsKey' => 'SupplierContracts',
				'stats' => array('suppliercontract_inactive', 'suppliercontract_active'),
			);
		}

		$this->results = $dashboardgroup;
		return 0;
	}

	/**
	 * Overload the setLinkedObjectSourceTargetType function : replacing the parent's function with the one below
	 *
	 * @param	array<string,mixed>	$parameters		Hook metadata (context, etc...)
	 * @param	CommonObject		$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	?string				$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager			$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int									Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function setLinkedObjectSourceTargetType($parameters, &$object, &$action, $hookmanager)
	{
		$contexts = explode(':', $parameters['context']);

		if (in_array('suppliercontractcard', $contexts)) {
			$targettype = $parameters['targettype'];

			if ($targettype == 'suppliercontract_suppliercontract') {
				$this->results = ['targettype' => 'suppliercontract'];
				return 1;
			}
		}

		return 0;
	}

	/**
	 * Overload the addMoreRecentObjects function : replacing the parent's function with the one below
	 *
	 * @param	array<string,mixed>	$parameters		Hook metadata (context, etc...)
	 * @param	CommonObject		$object			The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param	?string				$action			Current action (if set). Generally create or edit or null
	 * @param	HookManager			$hookmanager	Hook manager propagated to allow calling another hook
	 * @return	int									Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function addMoreRecentObjects($parameters, &$object, &$action, $hookmanager)
	{
		global $conf, $langs, $user;

		$contexts = explode(':', $parameters['context']);

		if (in_array('thirdpartysupplier', $contexts)) {
			$MAXLIST = getDolGlobalString('MAIN_SIZE_SHORTLIST_LIMIT');

			/*
			 * Latest supplier contracts
			 */
			if (isModEnabled('suppliercontract') && $user->hasRight('suppliercontract', 'lire')) {
				$sql = "SELECT s.nom, s.rowid, c.rowid as id, c.ref as ref, c.fk_project, c.status as contract_status, c.date_creation as dc, c.date_contract as dcon, c.ref_customer as refcus, c.ref_supplier as refsup, c.entity,";
				$sql .= " c.last_main_doc, c.model_pdf";
				$sql .= " FROM " . MAIN_DB_PREFIX . "societe as s, " . MAIN_DB_PREFIX . "suppliercontract as c";
				$sql .= " WHERE c.fk_soc = s.rowid ";
				$sql .= " AND s.rowid = " . ((int)$object->id);
				$sql .= " AND c.entity IN (" . getEntity('suppliercontract') . ")";
				$sql .= " ORDER BY c.date_creation DESC";

				$resql = $this->db->query($sql);
				if ($resql) {
					$contrat = new SupplierContract($this->db);
					$formfile = new FormFile($this->db);
					$project = new Project($this->db);
					$now = dol_now();

					$num = $this->db->num_rows($resql);
					if ($num > 0) {
						print '<div class="div-table-responsive-no-min">';
						print '<table class="noborder centpercent lastrecordtable">';

						print '<tr class="liste_titre">';
						print '<td colspan="6"><table width="100%" class="nobordernopadding"><tr><td>' . $langs->trans("LastSupplierContracts", ($num <= $MAXLIST ? "" : $MAXLIST)) . '</td>';
						print '<td class="right"><a class="notasortlink" href="' . dol_buildpath('/suppliercontract/list.php', 1) . '?socid=' . $object->id . '">' . $langs->trans("AllSupplierContracts") . '<span class="badge marginleftonlyshort">' . $num . '</span></a></td>';
						//print '<td width="20px" class="right"><a href="'.dol_buildpath('/suppliercontract/stats/index.php', 1).'?socid='.$object->id.'">'.img_picto($langs->trans("Statistics"),'stats').'</a></td>';
						print '</tr></table></td>';
						print '</tr>';
					}

					$i = 0;
					while ($i < $num && $i < $MAXLIST) {
						$objp = $this->db->fetch_object($resql);

						$contrat->id = $objp->id;
						$contrat->ref = $objp->ref ? $objp->ref : $objp->id;
						$contrat->ref_customer = $objp->refcus;
						$contrat->ref_supplier = $objp->refsup;
						$contrat->fk_project = $objp->fk_project;
						$contrat->status = $objp->contract_status;
						$contrat->last_main_doc = $objp->last_main_doc;
						$contrat->model_pdf = $objp->model_pdf;
						$contrat->fetch_lines();

						$late = '';
						foreach ($contrat->lines as $line) {
							if ($contrat->status == SupplierContract::STATUS_VALIDATED && $line->status == SupplierContractLine::STATUS_OPEN) {
								if (((!empty($line->date_end) ? $line->date_end : 0) + getDolGlobalInt('MAIN_DELAY_RUNNING_SUPPLIER_SERVICES')/*$conf->suppliercontract->services->expires->warning_delay*/) < $now) {
									$late = img_warning($langs->trans("Late"));
								}
							}
						}

						print '<tr class="oddeven">';
						print '<td class="nowraponall">';
						print $contrat->getNomUrl(1, 12);
						if (!empty($contrat->model_pdf)) {
							// Preview
							$filedir = $conf->suppliercontract->multidir_output[$objp->entity] . '/' . dol_sanitizeFileName($objp->ref);
							$file_list = null;
							if (!empty($filedir)) {
								$file_list = dol_dir_list($filedir, 'files', 0, '', '(\.meta|_preview.*.*\.png)$', 'date', SORT_DESC);
							}
							if (is_array($file_list)) {
								// Defined relative dir to DOL_DATA_ROOT
								$relativedir = '';
								if ($filedir) {
									$relativedir = preg_replace('/^' . preg_quote(DOL_DATA_ROOT, '/') . '/', '', $filedir);
									$relativedir = preg_replace('/^[\\/]/', '', $relativedir);
								}
								// Get list of files stored into database for same relative directory
								if ($relativedir) {
									completeFileArrayWithDatabaseInfo($file_list, $relativedir);

									//var_dump($sortfield.' - '.$sortorder);
									if (!empty($sortfield) && !empty($sortorder)) {    // If $sortfield is for example 'position_name', we will sort on the property 'position_name' (that is concat of position+name)
										$file_list = dol_sort_array($file_list, $sortfield, $sortorder);
									}
								}
								$relativepath = dol_sanitizeFileName($objp->ref) . '/' . dol_sanitizeFileName($objp->ref) . '.pdf';
								print $formfile->showPreview($file_list, $contrat->element, $relativepath, 0);
							}
						}
						// $filename = dol_sanitizeFileName($objp->ref);
						// $filedir = $conf->contrat->multidir_output[$objp->entity].'/'.dol_sanitizeFileName($objp->ref);
						// $urlsource = '/contrat/card.php?id='.$objp->cid;
						// print $formfile->getDocumentsLink($contrat->element, $filename, $filedir);
						print $late;
						print '</td><td class="tdoverflowmax100">';
						if ($contrat->fk_project > 0) {
							$project->fetch($contrat->fk_project);
							print $project->getNomUrl(1);
						}
						print "</td>\n";
						print '<td class="nowrap">';
						print dol_trunc(strtolower(get_class($object)) == strtolower(Client::class) ? $objp->refcus : $objp->refsup, 12);
						print "</td>\n";
						//print '<td class="right" width="80px"><span title="'.$langs->trans("DateCreation").'">'.dol_print_date($this->db->jdate($objp->dc), 'day')."</span></td>\n";
						print '<td class="right" width="80px"><span title="' . $langs->trans("DateContract") . '">' . dol_print_date($this->db->jdate($objp->dcon), 'day') . "</span></td>\n";
						print '<td width="20">&nbsp;</td>';
						print '<td class="nowraponall right">';
						print $contrat->getLibStatut(4);
						print "</td>\n";
						print '</tr>';
						$i++;
					}
					$this->db->free($resql);

					if ($num > 0) {
						print "</table>";
						print '</div>';
					}
				} else {
					dol_print_error($this->db);
				}
			}
		}

		return 0;
	}
}
