<?php
/* Copyright (C) 2004-2018	Laurent Destailleur			<eldy@users.sourceforge.net>
 * Copyright (C) 2018-2019	Nicolas ZABOURI				<info@inovea-conseil.com>
 * Copyright (C) 2019-2024	Frédéric France				<frederic.france@free.fr>
 * Copyright (C) 2024      	Easya Solutions        		<support@easya.solutions>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 *  \defgroup   suppliercontract     Module Supplier Contract
 *  \brief      Manage Supplier Contract service.
 *
 *  \file       htdocs/suppliercontract/core/modules/modSupplierContract.class.php
 *  \ingroup    suppliercontract
 *  \brief      Description and activation file for module Supplier Contract
 */

include_once DOL_DOCUMENT_ROOT . '/core/modules/DolibarrModules.class.php';


/**
 *  Description and activation class for module Supplier Contract
 */
class modSupplierContract extends DolibarrModules
{
	/**
	 * Constructor. Define names, constants, directories, boxes, permissions
	 *
	 * @param DoliDB $db Database handler
	 */
	public function __construct($db)
	{
		global $conf, $langs;

		$this->db = $db;

		// Id for module (must be unique).
		// Use here a free id (See in Home -> System information -> Dolibarr for list of used modules id).
		$this->numero = 163106; // TODO Go on page https://wiki.dolibarr.org/index.php/List_of_modules_id to reserve an id number for your module

		// Key text used to identify module (for permissions, menus, etc...)
		$this->rights_class = 'suppliercontract';

		// Family can be 'base' (core modules),'crm','financial','hr','projects','products','ecm','technic' (transverse modules),'interface' (link with external tools),'other','...'
		// It is used to group modules by family in module setup page
		$this->family = "easya";

		// Module position in the family on 2 digits ('01', '10', '20', ...)
		$this->module_position = '500';

		// Gives the possibility for the module, to provide his own family info and position of this family (Overwrite $this->family and $this->module_position. Avoid this)
		$this->familyinfo = array('easya' => array('position' => '001', 'label' => $langs->trans("easyaFamily")));
		// Module label (no space allowed), used if translation string 'ModuleMyModuleName' not found (MyModule is name of module).
		$this->name = preg_replace('/^mod/i', '', get_class($this));

		// DESCRIPTION_FLAG
		// Module description, used if translation string 'ModuleMyModuleDesc' not found (MyModule is name of module).
		$this->description = "Description of module Supplier Contract";
		// Used only if file README.md and README-LL.md not found.
		$this->descriptionlong = "";

		// Author
		$this->editor_name = '<b>Easya Solutions</b>';
		$this->editor_url = 'https://easya.solutions';        // Must be an external online web site
		$this->editor_squarred_logo = '';                    // Must be image filename into the module/img directory followed with @modulename. Example: 'myimage.png@mymodule'
		$this->editor_web = 'https://easya.solutions';
		$this->editor_email = 'support@easya.solutions';

		// Possible values for version are: 'development', 'experimental', 'dolibarr', 'dolibarr_deprecated', 'experimental_deprecated' or a version string like 'x.y.z'
		$this->version = trim(file_get_contents(__DIR__ . '/../../VERSION'));
		// Url to the file with your last numberversion of this module
		$this->url_last_version = 'https://git.open-dsi.fr/dolibarr-extension/' . strtolower($this->name) . '/-/raw/2024/VERSION';

		// Key used in llx_const table to save module status enabled/disabled (where MYMODULE is value of property name of module in uppercase)
		$this->const_name = 'MAIN_MODULE_' . strtoupper($this->name);

		// Name of image file used for this module.
		// If file is in theme/yourtheme/img directory under name object_pictovalue.png, use this->picto='pictovalue'
		// If file is in module/img directory under name object_pictovalue.png, use this->picto='pictovalue@module'
		// To use a supported fa-xxx css style of font awesome, use this->picto='xxx'
		$this->picto = 'easya_big@' . strtolower($this->name);

		// Define some features supported by module (triggers, login, substitutions, menus, css, etc...)
		$this->module_parts = array(
			// Set this to 1 if module has its own trigger directory (core/triggers)
			//'triggers' => 0,
			// Set this to 1 if module has its own login method file (core/login)
			//'login' => 0,
			// Set this to 1 if module has its own substitution function file (core/substitutions)
			'substitutions' => 1,
			// Set this to 1 if module has its own menus handler directory (core/menus)
			//'menus' => 0,
			// Set this to 1 if module overwrite template dir (core/tpl)
			'tpl' => 1,
			// Set this to 1 if module has its own barcode directory (core/modules/barcode)
			//'barcode' => 0,
			// Set this to 1 if module has its own models directory (core/modules/xxx)
			'models' => 1,
			// Set this to 1 if module has its own printing directory (core/modules/printing)
			//'printing' => 0,
			// Set this to 1 if module has its own theme directory (theme)
			//'theme' => 0,
			// Set this to relative path of css file if module has its own css file
			//'css' => array(//    '/mymodule/css/mymodule.css.php',
			//),
			// Set this to relative path of js file if module must load a js on all pages
			//'js' => array(//   '/mymodule/js/mymodule.js.php',
			//),
			// Set here all hooks context managed by module. To find available hook context, make a "grep -r '>initHooks(' *" on source code. You can also set hook context to 'all'
			/* BEGIN MODULEBUILDER HOOKSCONTEXTS */
			'hooks' => array(
				'data' => array(
					'searchform',
					'projectOverview',
					'index',
					'commonobject',
					'completeListOfReferent',
					'thirdpartysupplier',
					'projectcard',
					'supplier_proposalcard',
					'ordersuppliercard',
					'invoicesuppliercard',
					'suppliercontractcard',
				),
				//'entity' => '0',
			),
			/* END MODULEBUILDER HOOKSCONTEXTS */
			// Set this to 1 if features of module are opened to external users
			//'moduleforexternal' => 0,
			// Set this to 1 if the module provides a website template into doctemplates/websites/website_template-mytemplate
			//'websitetemplates' => 0
		);

		// Data directories to create when module is enabled.
		// Example: this->dirs = array("/mymodule/temp","/mymodule/subdir");
		$this->dirs = array('/suppliercontract');

		// Config pages. Put here list of php page, stored into mymodule/admin directory, to use to setup module.
		$this->config_page_url = array("suppliercontract.php@suppliercontract");

		// Dependencies
		// A condition to hide module
		$this->hidden = getDolGlobalInt('MODULE_SUPPLIERCONTRACT_DISABLED'); // A condition to disable module;
		// List of module class names that must be enabled if this module is enabled. Example: array('always'=>array('modModuleToEnable1','modModuleToEnable2'), 'FR'=>array('modModuleToEnableFR')...)
		$this->depends = array("modFournisseur");
		// List of module class names to disable if this one is disabled. Example: array('modModuleToDisable1', ...)
		$this->requiredby = array();
		// List of module class names this module is in conflict with. Example: array('modModuleToDisable1', ...)
		$this->conflictwith = array();

		// The language file dedicated to your module
		$this->langfiles = array("suppliercontracts@suppliercontract", "opendsi@suppliercontract");

		// Prerequisites
		$easya_info = json_decode(file_get_contents(__DIR__ . '/../../.easya_info.json'));
		$this->phpmin = explode('.', $easya_info->php_min_version); // Minimum version of PHP required by module
		$this->need_dolibarr_version = explode('.', $easya_info->dlb_min_version); // Minimum version of Dolibarr required by module
		$this->need_javascript_ajax = 0;

		// Messages at activation
		$this->warnings_activation = array(); // Warning to show when we activate module. array('always'='text') or array('FR'='textfr','MX'='textmx'...)
		$this->warnings_activation_ext = array(); // Warning to show when we activate an external module. array('always'='text') or array('FR'='textfr','MX'='textmx'...)
		//$this->automatic_activation = array('FR'=>'MyModuleWasAutomaticallyActivatedBecauseOfYourCountryChoice');
		//$this->always_enabled = true;								// If true, can't be disabled

		// Constants
		// List of particular constants to add when module is enabled (key, 'chaine', value, desc, visible, 'current' or 'allentities', deleteonunactive)
		// Example: $this->const=array(1 => array('MYMODULE_MYNEWCONST1', 'chaine', 'myvalue', 'This is a constant to add', 1),
		//                             2 => array('MYMODULE_MYNEWCONST2', 'chaine', 'myvalue', 'This is another constant to add', 0, 'current', 1)
		// );
		$this->const = array();
		$r = 0;

		$this->const[$r][0] = "SUPPLIERCONTRACT_ADDON";
		$this->const[$r][1] = "chaine";
		$this->const[$r][2] = "mod_suppliercontract_serpis";
		$this->const[$r][3] = 'Nom du gestionnaire de numerotation des contrats fournisseurs';
		$this->const[$r][4] = 0;
		$r++;

		$this->const[$r][0] = "SUPPLIERCONTRACT_ADDON_PDF";
		$this->const[$r][1] = "chaine";
		$this->const[$r][2] = "suppliercontract_strato";
		$this->const[$r][3] = 'Name of PDF model of supplier contract';
		$this->const[$r][4] = 0;
		$r++;

		$this->const[$r][0] = "SUPPLIERCONTRACT_ADDON_PDF_ODT_PATH";
		$this->const[$r][1] = "chaine";
		$this->const[$r][2] = "DOL_DATA_ROOT/doctemplates/suppliercontracts";
		$this->const[$r][3] = "";
		$this->const[$r][4] = 0;
		$r++;

		// Some keys to add into the overwriting translation tables
		/*$this->overwrite_translation = array(
			'en_US:ParentCompany'=>'Parent company or reseller',
			'fr_FR:ParentCompany'=>'Maison mère ou revendeur'
		)*/

		if (!isModEnabled("suppliercontract")) {
			$conf->suppliercontract = new stdClass();
			$conf->suppliercontract->enabled = 0;
		}

		// Array to add new pages in new tabs
		/* BEGIN MODULEBUILDER TABS */
		$this->tabs = array();
		/* END MODULEBUILDER TABS */
		// Example:
		// To add a new tab identified by code tabname1
		// $this->tabs[] = array('data' => 'objecttype:+tabname1:Title1:mylangfile@mymodule:$user->hasRight(\'mymodule\', \'read\'):/mymodule/mynewtab1.php?id=__ID__');
		// To add another new tab identified by code tabname2. Label will be result of calling all substitution functions on 'Title2' key.
		// $this->tabs[] = array('data' => 'objecttype:+tabname2:SUBSTITUTION_Title2:mylangfile@mymodule:$user->hasRight(\'othermodule\', \'read\'):/mymodule/mynewtab2.php?id=__ID__',
		// To remove an existing tab identified by code tabname
		// $this->tabs[] = array('data' => 'objecttype:-tabname:NU:conditiontoremove');
		//
		// Where objecttype can be
		// 'categories_x'	  to add a tab in category view (replace 'x' by type of category (0=product, 1=supplier, 2=customer, 3=member)
		// 'contact'          to add a tab in contact view
		// 'contract'         to add a tab in contract view
		// 'delivery'         to add a tab in delivery view
		// 'group'            to add a tab in group view
		// 'intervention'     to add a tab in intervention view
		// 'invoice'          to add a tab in customer invoice view
		// 'invoice_supplier' to add a tab in supplier invoice view
		// 'member'           to add a tab in foundation member view
		// 'opensurveypoll'	  to add a tab in opensurvey poll view
		// 'order'            to add a tab in sale order view
		// 'order_supplier'   to add a tab in supplier order view
		// 'payment'		  to add a tab in payment view
		// 'payment_supplier' to add a tab in supplier payment view
		// 'product'          to add a tab in product view
		// 'propal'           to add a tab in propal view
		// 'project'          to add a tab in project view
		// 'stock'            to add a tab in stock view
		// 'thirdparty'       to add a tab in third party view
		// 'user'             to add a tab in user view


		// Dictionaries
		/* Example:
		$this->dictionaries=array(
		 'langs' => 'mymodule@mymodule',
		 // List of tables we want to see into dictonnary editor
		 'tabname' => array("table1", "table2", "table3"),
		 // Label of tables
		 'tablib' => array("Table1", "Table2", "Table3"),
		 // Request to select fields
		 'tabsql' => array('SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table1 as f', 'SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table2 as f', 'SELECT f.rowid as rowid, f.code, f.label, f.active FROM '.MAIN_DB_PREFIX.'table3 as f'),
		 // Sort order
		 'tabsqlsort' => array("label ASC", "label ASC", "label ASC"),
		 // List of fields (result of select to show dictionary)
		 'tabfield' => array("code,label", "code,label", "code,label"),
		 // List of fields (list of fields to edit a record)
		 'tabfieldvalue' => array("code,label", "code,label", "code,label"),
		 // List of fields (list of fields for insert)
		 'tabfieldinsert' => array("code,label", "code,label", "code,label"),
		 // Name of columns with primary key (try to always name it 'rowid')
		 'tabrowid' => array("rowid", "rowid", "rowid"),
		 // Condition to show each dictionary
		 'tabcond' => array(isModEnabled('mymodule'), isModEnabled('mymodule'), isModEnabled('mymodule')),
		 // Tooltip for every fields of dictionaries: DO NOT PUT AN EMPTY ARRAY
		 'tabhelp' => array(array('code' => $langs->trans('CodeTooltipHelp'), 'field2' => 'field2tooltip'), array('code' => $langs->trans('CodeTooltipHelp'), 'field2' => 'field2tooltip'), ...),
		);
		*/
		/* BEGIN MODULEBUILDER DICTIONARIES */
		$this->dictionaries = array();
		/* END MODULEBUILDER DICTIONARIES */

		// Boxes/Widgets
		// Add here list of php file(s) stored in mymodule/core/boxes that contains a class to show a widget.
		/* BEGIN MODULEBUILDER WIDGETS */
		$this->boxes = array(
			//  0 => array(
			//      'file' => 'mymodulewidget1.php@mymodule',
			//      'note' => 'Widget provided by MyModule',
			//      'enabledbydefaulton' => 'Home',
			//  ),
			//  ...
			0 => array('file' => 'box_suppliercontracts.php', 'enabledbydefaulton' => 'Home'),
			1 => array('file' => 'box_supplier_services_expired.php', 'enabledbydefaulton' => 'Home'),
			//2 => array('file' => 'box_supplier_services_contracts.php', 'enabledbydefaulton' => 'Home')
		);
		/* END MODULEBUILDER WIDGETS */

		// Cronjobs (List of cron jobs entries to add when module is enabled)
		// unit_frequency must be 60 for minute, 3600 for hour, 86400 for day, 604800 for week
		/* BEGIN MODULEBUILDER CRON */
		$this->cronjobs = array(
			//  0 => array(
			//      'label' => 'MyJob label',
			//      'jobtype' => 'method',
			//      'class' => '/mymodule/class/myobject.class.php',
			//      'objectname' => 'MyObject',
			//      'method' => 'doScheduledJob',
			//      'parameters' => '',
			//      'comment' => 'Comment',
			//      'frequency' => 2,
			//      'unitfrequency' => 3600,
			//      'status' => 0,
			//      'test' => 'isModEnabled("mymodule")',
			//      'priority' => 50,
			//  ),
		);
		/* END MODULEBUILDER CRON */
		// Example: $this->cronjobs=array(
		//    0=>array('label'=>'My label', 'jobtype'=>'method', 'class'=>'/dir/class/file.class.php', 'objectname'=>'MyClass', 'method'=>'myMethod', 'parameters'=>'param1, param2', 'comment'=>'Comment', 'frequency'=>2, 'unitfrequency'=>3600, 'status'=>0, 'test'=>'isModEnabled("mymodule")', 'priority'=>50),
		//    1=>array('label'=>'My label', 'jobtype'=>'command', 'command'=>'', 'parameters'=>'param1, param2', 'comment'=>'Comment', 'frequency'=>1, 'unitfrequency'=>3600*24, 'status'=>0, 'test'=>'isModEnabled("mymodule")', 'priority'=>50)
		// );

		// Permissions provided by this module
		$this->rights = array();
		$r = 0;
		// Add here entries to declare new permissions
		/* BEGIN MODULEBUILDER PERMISSIONS */
		/*
		$o = 1;
		$this->rights[$r][0] = $this->numero . sprintf("%02d", ($o * 10) + 1); // Permission id (must not be already used)
		$this->rights[$r][1] = 'Read objects of MyModule'; // Permission label
		$this->rights[$r][4] = 'myobject';
		$this->rights[$r][5] = 'read'; // In php code, permission will be checked by test if ($user->hasRight('mymodule', 'myobject', 'read'))
		$r++;
		$this->rights[$r][0] = $this->numero . sprintf("%02d", ($o * 10) + 2); // Permission id (must not be already used)
		$this->rights[$r][1] = 'Create/Update objects of MyModule'; // Permission label
		$this->rights[$r][4] = 'myobject';
		$this->rights[$r][5] = 'write'; // In php code, permission will be checked by test if ($user->hasRight('mymodule', 'myobject', 'write'))
		$r++;
		$this->rights[$r][0] = $this->numero . sprintf("%02d", ($o * 10) + 3); // Permission id (must not be already used)
		$this->rights[$r][1] = 'Delete objects of MyModule'; // Permission label
		$this->rights[$r][4] = 'myobject';
		$this->rights[$r][5] = 'delete'; // In php code, permission will be checked by test if ($user->hasRight('mymodule', 'myobject', 'delete'))
		$r++;
		*/
		/* END MODULEBUILDER PERMISSIONS */
		$r++;
		$this->rights[$r][0] = 163157;
		$this->rights[$r][1] = 'Lire les contrats fournisseurs';
		$this->rights[$r][2] = 'r';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'lire';

		$r++;
		$this->rights[$r][0] = 163158;
		$this->rights[$r][1] = 'Creer / modifier les contrats fournisseurs';
		$this->rights[$r][2] = 'w';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'creer';

		$r++;
		$this->rights[$r][0] = 163159;
		$this->rights[$r][1] = 'Activer un service d\'un contrat fournisseur';
		$this->rights[$r][2] = 'w';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'activer';

		$r++;
		$this->rights[$r][0] = 163160;
		$this->rights[$r][1] = 'Desactiver un service d\'un contrat fournisseur';
		$this->rights[$r][2] = 'w';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'desactiver';

		$r++;
		$this->rights[$r][0] = 163161;
		$this->rights[$r][1] = 'Supprimer un contrat fournisseur';
		$this->rights[$r][2] = 'd';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'supprimer';

		$r++;
		$this->rights[$r][0] = 163162;
		$this->rights[$r][1] = 'Export contracts fournisseurs';
		$this->rights[$r][2] = 'r';
		$this->rights[$r][3] = 0;
		$this->rights[$r][4] = 'export';


		// Main menu entries to add
		$this->menu = array();
		$r = 0;

		// Add here entries to declare new menus
		/* BEGIN MODULEBUILDER TOPMENU */
		/*
		$this->menu[$r++] = array(
			'fk_menu' => '', // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
			'type' => 'top', // This is a Top menu entry
			'titre' => 'ModuleMyModuleName',
			'prefix' => img_picto('', $this->picto, 'class="pictofixedwidth valignmiddle"'),
			'mainmenu' => 'mymodule',
			'leftmenu' => '',
			'url' => '/mymodule/mymoduleindex.php',
			'langs' => 'mymodule@mymodule', // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
			'position' => 1000 + $r,
			'enabled' => 'isModEnabled("mymodule")', // Define condition to show or hide menu entry. Use 'isModEnabled("mymodule")' if entry must be visible if module is enabled.
			'perms' => '1', // Use 'perms'=>'$user->hasRight("mymodule", "myobject", "read")' if you want your menu with a permission rules
			'target' => '',
			'user' => 2, // 0=Menu for internal users, 1=external users, 2=both
		);
		*/
		/* END MODULEBUILDER TOPMENU */

		/* BEGIN MODULEBUILDER LEFTMENU MYOBJECT */
		/*
		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=mymodule',      // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
			'type' => 'left',                          // This is a Left menu entry
			'titre' => 'MyObject',
			'prefix' => img_picto('', $this->picto, 'class="pictofixedwidth valignmiddle paddingright"'),
			'mainmenu' => 'mymodule',
			'leftmenu' => 'myobject',
			'url' => '/mymodule/mymoduleindex.php',
			'langs' => 'mymodule@mymodule',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
			'position' => 1000 + $r,
			'enabled' => 'isModEnabled("mymodule")', // Define condition to show or hide menu entry. Use 'isModEnabled("mymodule")' if entry must be visible if module is enabled.
			'perms' => '$user->hasRight("mymodule", "myobject", "read")',
			'target' => '',
			'user' => 2,				                // 0=Menu for internal users, 1=external users, 2=both
			'object' => 'MyObject'
		);
		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=mymodule,fk_leftmenu=myobject',	    // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
			'type' => 'left',			                // This is a Left menu entry
			'titre' => 'New_MyObject',
			'mainmenu' => 'mymodule',
			'leftmenu' => 'mymodule_myobject_new',
			'url' => '/mymodule/myobject_card.php?action=create',
			'langs' => 'mymodule@mymodule',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
			'position' => 1000 + $r,
			'enabled' => 'isModEnabled("mymodule")', // Define condition to show or hide menu entry. Use 'isModEnabled("mymodule")' if entry must be visible if module is enabled. Use '$leftmenu==\'system\'' to show if leftmenu system is selected.
			'perms' => '$user->hasRight("mymodule", "myobject", "write")'
			'target' => '',
			'user' => 2,				                // 0=Menu for internal users, 1=external users, 2=both
			'object' => 'MyObject'
		);
		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=mymodule,fk_leftmenu=myobject',	    // '' if this is a top menu. For left menu, use 'fk_mainmenu=xxx' or 'fk_mainmenu=xxx,fk_leftmenu=yyy' where xxx is mainmenucode and yyy is a leftmenucode
			'type' => 'left',			                // This is a Left menu entry
			'titre' => 'List_MyObject',
			'mainmenu' => 'mymodule',
			'leftmenu' => 'mymodule_myobject_list',
			'url' => '/mymodule/myobject_list.php',
			'langs' => 'mymodule@mymodule',	        // Lang file to use (without .lang) by module. File must be in langs/code_CODE/ directory.
			'position' => 1000 + $r,
			'enabled' => 'isModEnabled("mymodule")', // Define condition to show or hide menu entry. Use 'isModEnabled("mymodule")' if entry must be visible if module is enabled.
			'perms' => '$user->hasRight("mymodule", "myobject", "read")'
			'target' => '',
			'user' => 2,				                // 0=Menu for internal users, 1=external users, 2=both
			'object' => 'MyObject'
		);
		*/
		/* END MODULEBUILDER LEFTMENU MYOBJECT */

		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=commercial',
			'type' => 'left',
			'prefix' => img_picto('', 'contract', 'class="paddingright pictofixedwidth"'),
			'titre' => 'SupplierContractsSubscriptions',
			'mainmenu' => 'commercial',
			'leftmenu' => 'suppliercontract',
			'url' => '/suppliercontract/index.php?leftmenu=suppliercontracts',
			'langs' => 'suppliercontracts@suppliercontract',
			'position' => 3000 + $r,
			'enabled' => 'isModEnabled("suppliercontract")',
			'perms' => '$user->hasRight("suppliercontract", "lire")',
			'target' => '',
			'user' => 0,
		);

		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=commercial,fk_leftmenu=suppliercontract',
			'type' => 'left',
			'titre' => 'NewSupplierContractSubscription',
			'mainmenu' => 'commercial',
			'leftmenu' => 'suppliercontract_new',
			'url' => '/suppliercontract/card.php?action=create&amp;leftmenu=suppliercontracts',
			'langs' => 'suppliercontracts@suppliercontract',
			'position' => 3000 + $r,
			'enabled' => 'isModEnabled("suppliercontract")',
			'perms' => '$user->hasRight("suppliercontract", "creer")',
			'target' => '',
			'user' => 0,
		);

		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=commercial,fk_leftmenu=suppliercontract',
			'type' => 'left',
			'titre' => 'List',
			'mainmenu' => 'commercial',
			'leftmenu' => 'suppliercontract_list',
			'url' => '/suppliercontract/list.php?leftmenu=suppliercontracts',
			'langs' => '',
			'position' => 3000 + $r,
			'enabled' => 'isModEnabled("suppliercontract")',
			'perms' => '$user->hasRight("suppliercontract", "lire")',
			'target' => '',
			'user' => 0,
		);

		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=commercial,fk_leftmenu=suppliercontract',
			'type' => 'left',
			'titre' => 'MenuServices',
			'mainmenu' => 'commercial',
			'leftmenu' => 'suppliercontract_services_list',
			'url' => '/suppliercontract/services_list.php?leftmenu=suppliercontracts',
			'langs' => 'contracts',
			'position' => 3000 + $r,
			'enabled' => 'isModEnabled("suppliercontract")',
			'perms' => '$user->hasRight("suppliercontract", "lire")',
			'target' => '',
			'user' => 0,
		);

		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=commercial,fk_leftmenu=suppliercontract_services_list',
			'type' => 'left',
			'titre' => 'MenuInactiveServices',
			'mainmenu' => 'commercial',
			'leftmenu' => 'suppliercontract_services_list_inactive',
			'url' => '/suppliercontract/services_list.php?leftmenu=suppliercontracts&amp;search_status=0',
			'langs' => 'contracts',
			'position' => 3000 + $r,
			'enabled' => 'isModEnabled("suppliercontract")',
			'perms' => '$user->hasRight("suppliercontract", "lire")',
			'target' => '',
			'user' => 0,
		);

		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=commercial,fk_leftmenu=suppliercontract_services_list',
			'type' => 'left',
			'titre' => 'MenuRunningServices',
			'mainmenu' => 'commercial',
			'leftmenu' => 'suppliercontract_services_list_running',
			'url' => '/suppliercontract/services_list.php?leftmenu=suppliercontracts&amp;search_status=4',
			'langs' => 'contracts',
			'position' => 3000 + $r,
			'enabled' => 'isModEnabled("suppliercontract")',
			'perms' => '$user->hasRight("suppliercontract", "lire")',
			'target' => '',
			'user' => 0,
		);

		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=commercial,fk_leftmenu=suppliercontract_services_list',
			'type' => 'left',
			'titre' => 'MenuExpiredServices',
			'mainmenu' => '',
			'leftmenu' => 'suppliercontract_services_list_expired',
			'url' => '/suppliercontract/services_list.php?leftmenu=suppliercontracts&amp;search_status=4%26filter=expired',
			'langs' => 'contracts',
			'position' => 3000 + $r,
			'enabled' => 'isModEnabled("suppliercontract")',
			'perms' => '$user->hasRight("suppliercontract", "lire")',
			'target' => '',
			'user' => 0,
		);

		$this->menu[$r++]=array(
			'fk_menu' => 'fk_mainmenu=commercial,fk_leftmenu=suppliercontract_services_list',
			'type' => 'left',
			'titre' => 'MenuClosedServices',
			'mainmenu' => '',
			'leftmenu' => 'suppliercontract_services_list_closed',
			'url' => '/suppliercontract/services_list.php?leftmenu=suppliercontracts&amp;search_status=5',
			'langs' => 'contracts',
			'position' => 3000 + $r,
			'enabled' => 'isModEnabled("suppliercontract")',
			'perms' => '$user->hasRight("suppliercontract", "lire")',
			'target' => '',
			'user' => 0,
		);

		// Exports profiles provided by this module
		$r = 1;
		/* BEGIN MODULEBUILDER EXPORT MYOBJECT */
		/*
		$langs->load("mymodule@mymodule");
		$this->export_code[$r] = $this->rights_class.'_'.$r;
		$this->export_label[$r] = 'MyObjectLines';	// Translation key (used only if key ExportDataset_xxx_z not found)
		$this->export_icon[$r] = $this->picto;
		// Define $this->export_fields_array, $this->export_TypeFields_array and $this->export_entities_array
		$keyforclass = 'MyObject'; $keyforclassfile='/mymodule/class/myobject.class.php'; $keyforelement='myobject@mymodule';
		include DOL_DOCUMENT_ROOT.'/core/commonfieldsinexport.inc.php';
		//$this->export_fields_array[$r]['t.fieldtoadd']='FieldToAdd'; $this->export_TypeFields_array[$r]['t.fieldtoadd']='Text';
		//unset($this->export_fields_array[$r]['t.fieldtoremove']);
		//$keyforclass = 'MyObjectLine'; $keyforclassfile='/mymodule/class/myobject.class.php'; $keyforelement='myobjectline@mymodule'; $keyforalias='tl';
		//include DOL_DOCUMENT_ROOT.'/core/commonfieldsinexport.inc.php';
		$keyforselect='myobject'; $keyforaliasextra='extra'; $keyforelement='myobject@mymodule';
		include DOL_DOCUMENT_ROOT.'/core/extrafieldsinexport.inc.php';
		//$keyforselect='myobjectline'; $keyforaliasextra='extraline'; $keyforelement='myobjectline@mymodule';
		//include DOL_DOCUMENT_ROOT.'/core/extrafieldsinexport.inc.php';
		//$this->export_dependencies_array[$r] = array('myobjectline' => array('tl.rowid','tl.ref')); // To force to activate one or several fields if we select some fields that need same (like to select a unique key if we ask a field of a child to avoid the DISTINCT to discard them, or for computed field than need several other fields)
		//$this->export_special_array[$r] = array('t.field' => '...');
		//$this->export_examplevalues_array[$r] = array('t.field' => 'Example');
		//$this->export_help_array[$r] = array('t.field' => 'FieldDescHelp');
		$this->export_sql_start[$r]='SELECT DISTINCT ';
		$this->export_sql_end[$r]  =' FROM '.MAIN_DB_PREFIX.'mymodule_myobject as t';
		//$this->export_sql_end[$r]  .=' LEFT JOIN '.MAIN_DB_PREFIX.'mymodule_myobject_line as tl ON tl.fk_myobject = t.rowid';
		$this->export_sql_end[$r] .=' WHERE 1 = 1';
		$this->export_sql_end[$r] .=' AND t.entity IN ('.getEntity('myobject').')';
		$r++; */
		/* END MODULEBUILDER EXPORT MYOBJECT */
		$this->export_code[$r] = $this->rights_class . '_' . $r;
		$this->export_label[$r] = 'SupplierContractsAndLine'; // Translation key (used only if key ExportDataset_xxx_z not found)
		$this->export_icon[$r] = 'contract';
		$this->export_permission[$r] = array(array("suppliercontract", "export"));
		$this->export_fields_array[$r] = array('s.rowid' => "IdCompany", 's.nom' => 'CompanyName', 's.address' => 'Address', 's.zip' => 'Zip', 's.town' => 'Town', 'c.code' => 'CountryCode',
			's.email' => 'Email', 's.phone' => 'Phone', 's.siren' => 'ProfId1', 's.siret' => 'ProfId2', 's.ape' => 'ProfId3', 's.idprof4' => 'ProfId4', 's.code_compta' => 'CustomerAccountancyCode',
			's.code_compta_fournisseur' => 'SupplierAccountancyCode', 's.tva_intra' => 'VATIntra',
			'co.rowid' => "Id", 'co.ref' => "Ref", 'co.date_creation' => "DateCreation", 'co.date_contract' => "DateContract",
			'co.note_private' => "NotePrivate", 'co.note_public' => "NotePublic",
			'cod.rowid' => 'LineId', 'cod.label' => "LineLabel", 'cod.description' => "LineDescription", 'cod.price_ht' => "LineUnitPrice", 'cod.tva_tx' => "LineVATRate",
			'cod.qty' => "LineQty", 'cod.total_ht' => "LineTotalHT", 'cod.total_tva' => "LineTotalVAT", 'cod.total_ttc' => "LineTotalTTC",
			'cod.date_opening' => "DateStart", 'cod.date_planned_opening' => "DateStartPlanned", 'cod.date_expiry' => "DateEndPlanned", 'cod.date_cloture' => "DateEnd",
			'p.rowid' => 'ProductId', 'p.ref' => 'ProductRef', 'p.label' => 'ProductLabel');

		$this->export_entities_array[$r] = array('s.rowid' => "company", 's.nom' => 'company', 's.address' => 'company', 's.zip' => 'company',
			's.town' => 'company', 'c.code' => 'company', 's.email' => 'company', 's.phone' => 'company', 's.siren' => 'company', 's.siret' => 'company', 's.ape' => 'company',
			's.idprof4' => 'company', 's.code_compta' => 'company', 's.code_compta_fournisseur' => 'company', 's.tva_intra' => 'company',
			'co.rowid' => "suppliercontract", 'co.ref' => "suppliercontract", 'co.date_creation' => "suppliercontract", 'co.date_contract' => "suppliercontract",
			'co.note_private' => "suppliercontract", 'co.note_public' => "suppliercontract",
			'cod.rowid' => 'suppliercontract_line', 'cod.label' => "suppliercontract_line", 'cod.description' => "suppliercontract_line", 'cod.price_ht' => "suppliercontract_line", 'cod.tva_tx' => "suppliercontract_line",
			'cod.qty' => "suppliercontract_line", 'cod.total_ht' => "suppliercontract_line", 'cod.total_tva' => "suppliercontract_line", 'cod.total_ttc' => "suppliercontract_line",
			'cod.date_opening' => "suppliercontract_line", 'cod.date_planned_opening' => "suppliercontract_line", 'cod.date_expiry' => "suppliercontract_line", 'cod.date_cloture' => "suppliercontract_line",
			'p.rowid' => 'product', 'p.ref' => 'product', 'p.label' => 'product');

		$this->export_TypeFields_array[$r] = array('s.rowid' => "Numeric", 's.nom' => 'Text', 's.address' => 'Text', 's.zip' => 'Text', 's.town' => 'Text', 'c.code' => 'Text',
			's.email' => 'Text', 's.phone' => 'Text', 's.siren' => 'Text', 's.siret' => 'Text', 's.ape' => 'Text', 's.idprof4' => 'Text', 's.code_compta' => 'Text',
			's.code_compta_fournisseur' => 'Text', 's.tva_intra' => 'Text',
			'co.ref' => "Text", 'co.date_creation' => "Date", 'co.date_contract' => "Date",
			'co.note_private' => "Text", 'co.note_public' => "Text",
			'cod.label' => "Text", 'cod.description' => "Text", 'cod.price_ht' => "Numeric", 'cod.tva_tx' => "Numeric",
			'cod.qty' => "Numeric", 'cod.total_ht' => "Numeric", 'cod.total_tva' => "Numeric", 'cod.total_ttc' => "Numeric",
			'cod.date_opening' => "Date", 'cod.date_planned_opening' => "Date", 'cod.date_expiry' => "Date", 'cod.date_cloture' => "Date",
			'p.rowid' => 'List:product:label', 'p.ref' => 'Text', 'p.label' => 'Text');


		$keyforselect = 'suppliercontract';
		$keyforelement = 'suppliercontract';
		$keyforaliasextra = 'coextra';
		include DOL_DOCUMENT_ROOT . '/core/extrafieldsinexport.inc.php';
		$keyforselect = 'suppliercontractdet';
		$keyforelement = 'suppliercontract_line';
		$keyforaliasextra = 'codextra';
		include DOL_DOCUMENT_ROOT . '/core/extrafieldsinexport.inc.php';

		$this->export_sql_start[$r] = 'SELECT DISTINCT ';
		$this->export_sql_end[$r] = ' FROM ' . MAIN_DB_PREFIX . 'societe as s';
		$this->export_sql_end[$r] .= ' LEFT JOIN ' . MAIN_DB_PREFIX . 'c_country as c on s.fk_pays = c.rowid';
		$this->export_sql_end[$r] .= ' INNER JOIN ' . MAIN_DB_PREFIX . 'suppliercontract as co ON co.fk_soc = s.rowid';
		$this->export_sql_end[$r] .= ' INNER JOIN ' . MAIN_DB_PREFIX . 'suppliercontractdet as cod ON co.rowid = cod.fk_suppliercontract';
		$this->export_sql_end[$r] .= ' LEFT JOIN ' . MAIN_DB_PREFIX . 'product as p ON (cod.fk_product = p.rowid)';
		$this->export_sql_end[$r] .= ' LEFT JOIN ' . MAIN_DB_PREFIX . 'suppliercontract_extrafields as coextra on (co.rowid = coextra.fk_object)';
		$this->export_sql_end[$r] .= ' LEFT JOIN ' . MAIN_DB_PREFIX . 'suppliercontractdet_extrafields as codextra on (cod.rowid = codextra.fk_object)';
		$this->export_sql_end[$r] .= ' WHERE co.entity IN (' . getEntity('contract') . ')';

		// Imports profiles provided by this module
		$r = 1;
		/* BEGIN MODULEBUILDER IMPORT MYOBJECT */
		/*
		$langs->load("mymodule@mymodule");
		$this->import_code[$r] = $this->rights_class.'_'.$r;
		$this->import_label[$r] = 'MyObjectLines';	// Translation key (used only if key ExportDataset_xxx_z not found)
		$this->import_icon[$r] = $this->picto;
		$this->import_tables_array[$r] = array('t' => MAIN_DB_PREFIX.'mymodule_myobject', 'extra' => MAIN_DB_PREFIX.'mymodule_myobject_extrafields');
		$this->import_tables_creator_array[$r] = array('t' => 'fk_user_author'); // Fields to store import user id
		$import_sample = array();
		$keyforclass = 'MyObject'; $keyforclassfile='/mymodule/class/myobject.class.php'; $keyforelement='myobject@mymodule';
		include DOL_DOCUMENT_ROOT.'/core/commonfieldsinimport.inc.php';
		$import_extrafield_sample = array();
		$keyforselect='myobject'; $keyforaliasextra='extra'; $keyforelement='myobject@mymodule';
		include DOL_DOCUMENT_ROOT.'/core/extrafieldsinimport.inc.php';
		$this->import_fieldshidden_array[$r] = array('extra.fk_object' => 'lastrowid-'.MAIN_DB_PREFIX.'mymodule_myobject');
		$this->import_regex_array[$r] = array();
		$this->import_examplevalues_array[$r] = array_merge($import_sample, $import_extrafield_sample);
		$this->import_updatekeys_array[$r] = array('t.ref' => 'Ref');
		$this->import_convertvalue_array[$r] = array(
			't.ref' => array(
				'rule'=>'getrefifauto',
				'class'=>(!getDolGlobalString('MYMODULE_MYOBJECT_ADDON') ? 'mod_myobject_standard' : getDolGlobalString('MYMODULE_MYOBJECT_ADDON')),
				'path'=>"/core/modules/mymodule/".(!getDolGlobalString('MYMODULE_MYOBJECT_ADDON') ? 'mod_myobject_standard' : getDolGlobalString('MYMODULE_MYOBJECT_ADDON')).'.php',
				'classobject'=>'MyObject',
				'pathobject'=>'/mymodule/class/myobject.class.php',
			),
			't.fk_soc' => array('rule' => 'fetchidfromref', 'file' => '/societe/class/societe.class.php', 'class' => 'Societe', 'method' => 'fetch', 'element' => 'ThirdParty'),
			't.fk_user_valid' => array('rule' => 'fetchidfromref', 'file' => '/user/class/user.class.php', 'class' => 'User', 'method' => 'fetch', 'element' => 'user'),
			't.fk_mode_reglement' => array('rule' => 'fetchidfromcodeorlabel', 'file' => '/compta/paiement/class/cpaiement.class.php', 'class' => 'Cpaiement', 'method' => 'fetch', 'element' => 'cpayment'),
		);
		$this->import_run_sql_after_array[$r] = array();
		$r++; */
		/* END MODULEBUILDER IMPORT MYOBJECT */
	}

	/**
	 *        Function called when module is enabled.
	 *        The init function add constants, boxes, permissions and menus (defined in constructor) into Dolibarr database.
	 *        It also creates data directories
	 *
	 * @param string $options Options when enabling module ('', 'noboxes')
	 * @return     int<-1,1>            1 if OK, <=0 if KO
	 */
	public function init($options = '')
	{
		global $conf, $langs;

		// Create tables of module at module activation
		//$result = $this->_load_tables('/install/mysql/', 'mymodule');
		$result = $this->_load_tables('/suppliercontract/sql/');
		if ($result < 0) {
			return -1; // Do not activate module if error 'not allowed' returned when loading module SQL queries (the _load_table run sql with run_sql with the error allowed parameter set to 'default')
		}

		// Create extrafields during init
		//include_once DOL_DOCUMENT_ROOT.'/core/class/extrafields.class.php';
		//$extrafields = new ExtraFields($this->db);
		//$result0=$extrafields->addExtraField('mymodule_separator1', "Separator 1", 'separator', 1,  0, 'thirdparty',   0, 0, '', array('options'=>array(1=>1)), 1, '', 1, 0, '', '', 'mymodule@mymodule', 'isModEnabled("mymodule")');
		//$result1=$extrafields->addExtraField('mymodule_myattr1', "New Attr 1 label", 'boolean', 1,  3, 'thirdparty',   0, 0, '', '', 1, '', -1, 0, '', '', 'mymodule@mymodule', 'isModEnabled("mymodule")');
		//$result2=$extrafields->addExtraField('mymodule_myattr2', "New Attr 2 label", 'varchar', 1, 10, 'project',      0, 0, '', '', 1, '', -1, 0, '', '', 'mymodule@mymodule', 'isModEnabled("mymodule")');
		//$result3=$extrafields->addExtraField('mymodule_myattr3', "New Attr 3 label", 'varchar', 1, 10, 'bank_account', 0, 0, '', '', 1, '', -1, 0, '', '', 'mymodule@mymodule', 'isModEnabled("mymodule")');
		//$result4=$extrafields->addExtraField('mymodule_myattr4', "New Attr 4 label", 'select',  1,  3, 'thirdparty',   0, 1, '', array('options'=>array('code1'=>'Val1','code2'=>'Val2','code3'=>'Val3')), 1,'', -1, 0, '', '', 'mymodule@mymodule', 'isModEnabled("mymodule")');
		//$result5=$extrafields->addExtraField('mymodule_myattr5', "New Attr 5 label", 'text',    1, 10, 'user',         0, 0, '', '', 1, '', -1, 0, '', '', 'mymodule@mymodule', 'isModEnabled("mymodule")');

		// Permissions
		$this->remove($options);

		$sql = array();

		//ODT template
		$src = dol_buildpath('/suppliercontract/install/doctemplates/suppliercontracts/template_suppliercontract.odt', 0);
		//$src = DOL_DOCUMENT_ROOT . '/install/doctemplates/suppliercontract/template_suppliercontract.odt';
		$dirodt = DOL_DATA_ROOT . '/doctemplates/suppliercontracts';
		$dest = $dirodt . '/template_suppliercontract.odt';

		if (file_exists($src) && !file_exists($dest)) {
			require_once DOL_DOCUMENT_ROOT . '/core/lib/files.lib.php';
			dol_mkdir($dirodt);
			$result = dol_copy($src, $dest, '0', 0);
			if ($result < 0) {
				$langs->load("errors");
				$this->error = $langs->trans('ErrorFailToCopyFile', $src, $dest);
				return 0;
			}
		}

		$sql = array(
			"DELETE FROM " . MAIN_DB_PREFIX . "document_model WHERE nom = '" . $this->db->escape($this->const[1][2]) . "' AND type = 'suppliercontract' AND entity = " . ((int)$conf->entity),
			"INSERT INTO " . MAIN_DB_PREFIX . "document_model (nom, type, entity) VALUES('" . $this->db->escape($this->const[1][2]) . "', 'suppliercontract', " . ((int)$conf->entity) . ")"
		);

		return $this->_init($sql, $options);
	}

	/**
	 * Function called when module is disabled.
	 * Remove from database constants, boxes and permissions from Dolibarr database.
	 * Data directories are not deleted
	 *
	 * @param string $options Options when enabling module ('', 'noboxes')
	 * @return    int<-1,1>                1 if OK, <=0 if KO
	 */
	public function remove($options = '')
	{
		$sql = array();

		return $this->_remove($sql, $options);
	}
}
